package thunder.jbosscluster;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.testng.Assert;
import org.testng.annotations.Test;

public class JBossClusteredHAJNDIFailoverWithNewCtxTest {

	@Test
	public void testHAJNDIWithNewInitCtxForEveryCall() {

		// new InitialContext instance for every all, so the ha-jndi proxy is always up to date with cluster.
		// this is another way to achieve failover for ha-jndi.
		// but extra network load is needed.

		// 1. look up the clustered ejb
		// 2. look up the non clustered ejb
		// 3. sleep current thread 2 mins
		// 4. shutdown the server manually and wait the thread to continue
		// 5. look up the clustered ejb
		// 6. look up the non cluster ejb

		ClusteredEJBRemote clusteredRemote = null;
		NonClusteredEJBRemote nonClusteredRemote = null;

		// 1. look up the clustered ejb
		try {
			clusteredRemote = HAJNDILookup(ClusteredEJBRemote.class, "ClusteredEJBMappedName", getInitialContext());
			System.out.println("helloCluster = " + clusteredRemote.helloCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}
		// 2. look up the non clustered ejb
		try {
			nonClusteredRemote = HAJNDILookup(NonClusteredEJBRemote.class, "NonClusteredEJBMappedName", getInitialContext());
			System.out.println("helloNonCluster = " + nonClusteredRemote.helloNonCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}

		System.out.println("========================================");
		System.out.println("pls shutdown the server...");

		// 3. sleep current thread 2 mins

		try {
			Thread.currentThread().sleep(1 * 60 * 1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
			Assert.fail("Thread Interrupted");
		}
		System.out.println("========================================");

		// 5. look up the clustered ejb.
		// everything should be fine even if the server which ha-jndi proxy is connecting to has been shutdown,
		// because we are using the newly created ha-jndi proxy.
		try {
			clusteredRemote = HAJNDILookup(ClusteredEJBRemote.class, "ClusteredEJBMappedName", getInitialContext());
			System.out.println("helloCluster = " + clusteredRemote.helloCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}
		// 6. look up the non cluster ejb
		try {
			nonClusteredRemote = HAJNDILookup(NonClusteredEJBRemote.class, "NonClusteredEJBMappedName", getInitialContext());
			System.out.println("helloNonCluster = " + nonClusteredRemote.helloNonCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}
	}

	private static <T> T HAJNDILookup(Class<T> businessInterfaceClass, String jndiName, InitialContext ctx) throws NamingException {

		T remoteBusinessObject = null;
		remoteBusinessObject = (T)ctx.lookup(jndiName);
		return remoteBusinessObject;
	}

	private static InitialContext getInitialContext() {

		Hashtable jndiEvn = new Hashtable();
		jndiEvn.put(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
		jndiEvn.put(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
		jndiEvn.put(Context.PROVIDER_URL, "jnp://10.28.71.51:1100,jnp://169.254.25.129:1100");
		jndiEvn.put("jnp.disableDiscovery", "false");

		InitialContext ctx = null;
		try {
			ctx = new InitialContext(jndiEvn);
		} catch (NamingException e) {
			e.printStackTrace();
		}
		return ctx;

	}
}